# This file runs the simulations underlying Fig. 4b and Fig. A5 in the manuscript entitled "Bee diversity effects depend on functional complementarity and niche shifts" by J. Fruend et al., published in Ecology
# it simulates bee - flower visitation networks based on preferences calculated from cages with 1 and cages with >1 bee species, respectively, and calculates plant species coverage for each simulation


#--- a few explanations:
# loaded objects:
  # webs.exp is an array of dims 21 plant species (named) X 5 bee species (named) X number of cages
    #! note that the data provided are just an example derived from the original data, and not the original data itself
      # (the number of observations for one cage is doubled in the example dataset in order for the simulations (rarefaction) to run with a small dataset)
  # flabun.w is a matrix [plant species X cage] of flower numbers weighted by their overall "attractiveness" (i.e. visitation frequency per flower across all bee cages)
  # Treatment is a vector [length = N cages], indicating the bee community per cage, containing combinations of letters A-E indicating which of the five bee species was present
  # treat_sr: vector [length = N cages], indicating bee species richness
  # treat_ext3 is a vector [length = N cages], indicating the treatment category per cage ("bee" for all bee cages)
  # SRvis.c is a vector [length = N cages], giving plant species coverage (number of visited plant species corrected for sampling) per cage
  # SRvis.mnull is a vector [length = N cages], which is the mean of nullmodel (Patefield) values of number of visited plant species per cage

library(vegan)        # loads R-package 'vegan', needed for rarefaction (function 'rrarefy'); code run with vegan 2.0-5

load('Fruend_Supplement_ExampleData_forRscript.rdata')  # loads the example data to run the script (objects explained above)


#------ Improved the separation of fundamental and interactive plant complementarity
# new approach: first calculating nullwebarrays
Nrep <- 10        # number of replicates; is set to low value for fast calculation; should be set to 1000
beecages <- which(treat_ext3=="bee")
plantnames.web <- as.factor(dimnames(webs.exp)[[1]])
treat_mix <- treat_sr > 1

#--| mono-pref loop [n reps, j cages, i bees, no index for plants]
  #! simulation takes about 2 min; load if you need it fast!
  nullwebarray.monopref <- array(0,dim=c(dim(webs.exp),Nrep))
    dimnames(nullwebarray.monopref)[1:3] <- dimnames(webs.exp)
  flabun.w.mono <- sapply(LETTERS[1:5],function(x)rowMeans(flabun.w[,Treatment==x,drop=F],na.rm=T))
     flabun.w.mono[is.na(flabun.w.mono)] <- 0
  totalweb.mono <- apply(webs.exp[,,treat_sr==1],1:2,sum,na.rm=T)
  pref.mono <- totalweb.mono /  flabun.w.mono
    pref.mono[is.na(pref.mono)] <- 0
  for (n in 1:Nrep) {          # simulating visits data (probability of visit based on flabun and mono-pref) ; takes about 2 minutes
    for (j in beecages){
      for (i in which(colSums(webs.exp)[,j]>0)){
        nullwebarray.monopref[,i,j,n] <- table(sample(plantnames.web,sum(webs.exp[,i,j]),replace=T,prob=pref.mono[,i]*flabun.w[,j]))
      }
    }
  }
  # save(nullwebarray.monopref,file="nullwebarray.monopref.rdata")
# load("nullwebarray.monopref.rdata")
srvis.mono <- apply(nullwebarray.monopref, 3:4, function(web)sum(rowSums(web)>0))
srvis.c.mono <- srvis.mono / SRvis.mnull  # this works, because SRvis.mnull always has the same Nvis and flabun

#--| poly-pref loop [n reps, j cages, i bees, no index for plants]
  #! simulation takes about 2 min; load if you need it fast!
  nullwebarray.polypref <- array(0,dim=c(dim(webs.exp),Nrep))
    dimnames(nullwebarray.polypref)[1:3] <- dimnames(webs.exp)
  flabun.w.poly <- sapply(LETTERS[1:5],function(x)rowMeans(flabun.w[,grepl(x,Treatment) & treat_sr>1],na.rm=T))
     flabun.w.poly[is.na(flabun.w.poly)] <- 0
  totalweb.poly <- apply(webs.exp[,,treat_sr>1],1:2,sum,na.rm=T)
  for (n in 1:Nrep) {          # simulating visits data (probability of visit based on flabun and poly-pref) ; takes about 2 minutes
    # bee-based rarefaction  (using rrarefy from vegan package);  one sample per null model replicate; pref needs to be calculated in n-loop
    totalweb.poly.rf <- t(rrarefy(t(totalweb.poly), sample=colSums(totalweb.mono)))
    pref.poly <- totalweb.poly.rf /  flabun.w.poly
      pref.poly[is.na(pref.poly)] <- 0
    for (j in beecages){
      for (i in which(colSums(webs.exp)[,j]>0)){
        nullwebarray.polypref[,i,j,n] <- table(sample(plantnames.web,sum(webs.exp[,i,j]),replace=T,prob=pref.poly[,i]*flabun.w[,j]))
      }
    }
  }
  # save(nullwebarray.polypref,file="nullwebarray.polypref.rdata")
# load("nullwebarray.polypref.rdata")
srvis.poly <- apply(nullwebarray.polypref, 3:4, function(web)sum(rowSums(web)>0))
srvis.c.poly <- srvis.poly / SRvis.mnull  # this works, because SRvis.mnull always has the same Nvis and flabun


#--| preparing monopoly fundamental-realized comparison (mostly for the plots)
  # means and conf-int:
  values.mono <- apply(srvis.c.mono,2,function(x)tapply(x,treat_mix,mean))    #! srvis.c is used here!!
  values.poly <- apply(srvis.c.poly,2,function(x)tapply(x,treat_mix,mean))
  SRvismeans <- matrix(c(tapply(SRvis.c,treat_mix,mean),apply(values.mono,1,mean),apply(values.poly,1,mean)),byrow=F,nrow=2)
    dimnames(SRvismeans) <- list(c("mono","poly"),c("observed","mono exp.","poly exp."))
  confint.mono <- t(apply(values.mono,1,function(x)quantile(x,c(0.025,0.975))))
  confint.poly <- t(apply(values.poly,1,function(x)quantile(x,c(0.025,0.975))))
# plot with dots  [21plants]
par(mar=c(2,4,0.4,0.2),las=1)
xpos <- matrix(c(1:3,(1:3)+0.3),nrow=2,byrow=T)
plot(xpos[1,],SRvismeans["mono",],xpd=F,ylab="Mean of plant species coverage",bg=rgb(0.9,0.9,0.9,0.99),pch=21,cex=2,ylim=c(0.65,1.05),xlim=c(min(xpos)-0.1,max(xpos)+0.1),xaxt="n",bty="l")
  points(xpos[2,],SRvismeans["poly",],bg=rgb(0.3,0.3,0.3,0.99),pch=21,cex=2)
  lines(c(0,10),c(tapply(SRvis.c,treat_mix,mean)[1],tapply(SRvis.c,treat_mix,mean)[1]),lty=2)
  lines(c(0,10),c(tapply(SRvis.c,treat_mix,mean)[2],tapply(SRvis.c,treat_mix,mean)[2]),lty=2)
  lines(c(xpos[1,2],xpos[1,2]), c(confint.mono[1,1], confint.mono[1,2]))
  lines(c(xpos[2,2],xpos[2,2]), c(confint.mono[2,1], confint.mono[2,2]))
  lines(c(xpos[1,3],xpos[1,3]), c(confint.poly[1,1], confint.poly[1,2]))
  lines(c(xpos[2,3],xpos[2,3]), c(confint.poly[2,1], confint.poly[2,2]))
  points(xpos[1,],SRvismeans["mono",],bg=rgb(0.9,0.9,0.9,0.99),pch=21,cex=2)
  points(xpos[2,],SRvismeans["poly",],bg=rgb(0.3,0.3,0.3,0.99),pch=21,cex=2)
  abline(v=(xpos[2,1]+xpos[1,2])/2,lty=1)
